@@ -1,11 +1,9 @@
 package WWW::YoutubeViewer;
 
 use utf8;
-use 5.014;
+use 5.016;
 use warnings;
 
-no if $] >= 5.018, warnings => 'experimental::smartmatch';
-
 use parent qw(
   WWW::YoutubeViewer::Search
   WWW::YoutubeViewer::Videos
@@ -25,7 +23,7 @@ WWW::YoutubeViewer - A very easy interface to YouTube.
 
 =cut
 
-our $VERSION = '3.3.1';
+our $VERSION = '3.3.3';
 
 =head1 SYNOPSIS
 
@@ -44,7 +42,7 @@ my %valid_options = (
     v               => {valid => q[],                                                    default => 3},
     page            => {valid => [qr/^(?!0+\z)\d+\z/],                                   default => 1},
     http_proxy      => {valid => [qr{^https?://}],                                       default => undef},
-    hl              => {valid => [qr/^\w+(?:[-_]\w+)?\z/],                               default => undef},
+    hl              => {valid => [qr/^\w+(?:[\-_]\w+)?\z/],                              default => undef},
     maxResults      => {valid => [1 .. 50],                                              default => 10},
     topicId         => {valid => [qr/^./],                                               default => undef},
     order           => {valid => [qw(relevance date rating viewCount title videoCount)], default => undef},
@@ -103,6 +101,28 @@ my %valid_options = (
     lwp_agent => {valid => [qr/^.{5}/], default => 'Mozilla/5.0 (X11; U; Linux i686; gzip; en-US) Chrome/10.0.648.45'},
 );
 
+sub _our_smartmatch {
+    my ($value, $arg) = @_;
+
+    $value // return 0;
+
+    if (ref($arg) eq '') {
+        return ($value eq $arg);
+    }
+
+    if (ref($arg) eq ref(qr//)) {
+        return scalar($value =~ $arg);
+    }
+
+    if (ref($arg) eq 'ARRAY') {
+        foreach my $item (@$arg) {
+            return 1 if __SUB__->($value, $item);
+        }
+    }
+
+    return 0;
+}
+
 {
     no strict 'refs';
 
@@ -114,7 +134,7 @@ my %valid_options = (
             *{__PACKAGE__ . '::set_' . $key} = sub {
                 my ($self, $value) = @_;
                 $self->{$key} =
-                    $value ~~ $valid_options{$key}{valid}
+                  _our_smartmatch($value, $valid_options{$key}{valid})
                   ? $value
                   : $valid_options{$key}{default};
             };
@@ -356,8 +376,13 @@ sub lwp_get {
         }
     }
 
-    if ($response->status_line() =~ /^500 /) {
-        return $self->lwp_get($url, %opt);    # try again
+    $opt{depth} ||= 0;
+
+    # Try again on 500+ HTTP errors
+    if (    $opt{depth} <= 3
+        and $response->code() >= 500
+        and $response->status_line() =~ /(?:Temporary|Server) Error|Timeout|Service Unavailable/i) {
+        return $self->lwp_get($url, %opt, depth => $opt{depth} + 1);
     }
 
     warn '[' . $response->status_line . "] Error occured on URL: $url\n";
@@ -469,69 +494,6 @@ sub _make_feed_url {
     $self->get_feeds_url() . $path . '?' . $self->default_arguments(%args);
 }
 
-=head2 get_courses_from_category($cat_id)
-
-Get the courses from a specific category ID.
-$cat_id can be any valid category ID from the EDU categories.
-
-=cut
-
-sub get_courses_from_category {
-    my ($self, $cat_id) = @_;
-
-    ...    # NEEDS WORK!!!
-}
-
-=head2 get_video_lectures_from_course($course_id)
-
-Get the video lectures from a specific course ID.
-$course_id can be any valid course ID from the EDU categories.
-
-=cut
-
-sub get_video_lectures_from_course {
-    my ($self, $course_id) = @_;
-
-    ...    # NEEDS WORK!!!
-}
-
-=head2 get_video_lectures_from_category($cat_id)
-
-Get the video lectures from a specific category ID.
-$cat_id can be any valid category ID from the EDU categories.
-
-=cut
-
-sub get_video_lectures_from_category {
-    my ($self, $cat_id) = @_;
-
-    ...    # NEEDS WORK!!!
-}
-
-=head2 get_movies($movieID)
-
-Get movie results for C<$movieID>.
-
-=cut
-
-sub get_movies {
-    my ($self, $movie_id) = @_;
-
-    ...    # NEEDS WORK!!!
-}
-
-=head2 get_video_tops(%opts)
-
-Returns the video tops for a specific feed_id.
-
-=cut
-
-sub get_video_tops {
-    my ($self, %opts) = @_;
-
-    ...    # NEEDS WORK!!!
-}
-
 sub _get_formats_from_ytdl {
     my ($self, $videoID) = @_;
 
@@ -635,18 +597,18 @@ sub _extract_streaming_urls {
     my %stream_map = $self->parse_query_string($info->{url_encoded_fmt_stream_map}, multi => 1);
     my %adaptive_fmts = $self->parse_query_string($info->{adaptive_fmts}, multi => 1);
 
-    my @result;
+    my @results;
 
-    push @result, $self->_group_keys_with_values(%stream_map);
-    push @result, $self->_group_keys_with_values(%adaptive_fmts);
+    push @results, $self->_group_keys_with_values(%stream_map);
+    push @results, $self->_group_keys_with_values(%adaptive_fmts);
 
-    foreach my $video (@result) {
+    foreach my $video (@results) {
         if (exists $video->{s}) {    # has an encrypted signature :(
 
             my @formats = $self->_get_formats_from_ytdl($videoID);
 
             foreach my $format (@formats) {
-                foreach my $ref (@result) {
+                foreach my $ref (@results) {
                     if (defined($ref->{itag}) and ($ref->{itag} eq $format->{itag})) {
                         $ref->{url} = $format->{url};
                         last;
@@ -659,15 +621,20 @@ sub _extract_streaming_urls {
     }
 
     if (exists $info->{hlsvp}) {
-        push @result,
-          {
-            itag => 38,
-            type => 'video/ts',
-            url  => $info->{hlsvp},
-          };
+        if (my @formats = $self->_get_formats_from_ytdl($videoID)) {
+            @results = @formats;
+        }
+        else {
+            push @results,
+              {
+                itag => 38,
+                type => 'video/ts',
+                url  => $info->{hlsvp},
+              };
+        }
     }
 
-    return @result;
+    return @results;
 }
 
 =head2 get_streaming_urls($videoID)
@@ -801,7 +768,7 @@ sub post_as_json {
 
 =head1 AUTHOR
 
-Trizen, C<< <trizenx at gmail.com> >>
+Trizen, C<< <echo dHJpemVuQHByb3Rvbm1haWwuY29tCg== | base64 -d> >>
 
 =head1 SEE ALSO
 
